@@ -48,6 +48,25 @@ def group_response(response, registration_ids, key):
     return grouping
 
 
+def urlencode_utf8(params):
+    """
+    UTF-8 safe variant of urllib.urlencode.
+    http://stackoverflow.com/a/8152242
+    """
+
+    if hasattr(params, 'items'):
+        params = params.items()
+
+    params =  (
+        '='.join((
+            urllib.quote_plus(k.encode('utf8'), safe='/'),
+            urllib.quote_plus(v.encode('utf8'), safe='/')
+        )) for k, v in params
+    )
+
+    return '&'.join(params)
+
+
 class GCM(object):
 
     # Timeunit is milliseconds.
@@ -94,14 +113,15 @@ class GCM(object):
         else:
             payload = {'registration_id': registration_ids}
             if data:
-                for k in data.keys():
-                    data['data.%s' % k] = data.pop(k)
-                payload.update(data)
+                plaintext_data = data.copy()
+                for k in plaintext_data.keys():
+                    plaintext_data['data.%s' % k] = plaintext_data.pop(k)
+                payload.update(plaintext_data)
 
         if delay_while_idle:
             payload['delay_while_idle'] = delay_while_idle
 
-        if time_to_live:
+        if time_to_live >= 0:
             payload['time_to_live'] = time_to_live
             if collapse_key is None:
                 raise GCMNoCollapseKeyException("collapse_key is required when time_to_live is provided")
@@ -132,7 +152,7 @@ class GCM(object):
             headers['Content-Type'] = 'application/json'
 
         if not is_json:
-            data = urllib.urlencode(data)
+            data = urlencode_utf8(data)
         req = urllib2.Request(self.url, data, headers)
 
         try:
@@ -144,6 +164,9 @@ class GCM(object):
                 raise GCMAuthenticationException("There was an error authenticating the sender account")
             elif e.code == 503:
                 raise GCMUnavailableException("GCM service is unavailable")
+            else:
+                error = "GCM service error: %d" % e.code
+                raise GCMUnavailableException(error)
         except urllib2.URLError as e:
             raise GCMConnectionException("There was an internal error in the GCM server while trying to process the request")
 
@@ -176,8 +199,7 @@ class GCM(object):
         else:
             if len(response_lines) == 2:
                 return response_lines[1].split('=')[1]
-            else:
-                return
+            return
 
     def handle_json_response(self, response, registration_ids):
         errors = group_response(response, registration_ids, 'error')
@@ -194,8 +216,7 @@ class GCM(object):
     def extract_unsent_reg_ids(self, info):
         if 'errors' in info and 'Unavailable' in info['errors']:
             return info['errors']['Unavailable']
-        else:
-            return []
+        return []
 
     def plaintext_request(self, registration_id, data=None, collapse_key=None,
                             delay_while_idle=False, time_to_live=None, retries=5):
